/***************************************************************************
                          tea_gtk_utils.c  -  description
                             -------------------
    begin                : aug. 2004
    copyright            : (C) 2004 by Peter 'Roxton' Semiletov
    email                : tea@list.ru
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/


#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif


#include <glib.h>
#include <gtk/gtk.h>
#include <unistd.h>
#include <sys/stat.h>
#include <gdk/gdkkeysyms.h>

#include <stddef.h>
#include <stdlib.h>
#include <unistd.h>
#include <sys/types.h>
#include <sys/wait.h>

#include "tea_defs.h"


GtkWidget* tea_text_entry (GtkWidget *container, gchar *caption, gchar *value)
{
  GtkWidget *b = gtk_hbox_new (FALSE, 1);
  gtk_widget_show (b);
  gtk_box_pack_start (container, b, FALSE, TRUE, 1);
 
  GtkWidget *x = gtk_label_new (caption);  
  gtk_widget_show (x);
  gtk_box_pack_start (b, x, FALSE, FALSE, 1);
  x = gtk_entry_new ();   
  gtk_widget_show (x);
  gtk_box_pack_start (b, x, TRUE, TRUE, 1);

  if (value)
     gtk_entry_set_text (x, value);

  return x;
}


static void cb_select_font (GtkObject *object, GtkWidget *w) 
{
  GtkEntry *e = w;

  GtkWidget *dlg_font_selector = gtk_font_selection_dialog_new (_("Choose a font"));
  
  gtk_font_selection_dialog_set_font_name (dlg_font_selector, gtk_entry_get_text (e));

  gchar *t;

  if (gtk_dialog_run (dlg_font_selector) == GTK_RESPONSE_OK)
     {
      t = gtk_font_selection_dialog_get_font_name (dlg_font_selector); 
      gtk_entry_set_text (e, t);  
      g_free (t);
     }

  gtk_widget_destroy (dlg_font_selector);
}


GtkWidget* tea_font_selector (GtkWidget *container, gchar *caption, gchar *value)
{

  GtkWidget *b = gtk_hbox_new (FALSE, 1);
  gtk_widget_show (b);
  gtk_box_pack_start (container, b, FALSE, TRUE, 1);

  GtkWidget *x = gtk_label_new (caption);  
  gtk_widget_show (x);
  gtk_box_pack_start (b, x, FALSE, FALSE, 1);

  x = gtk_entry_new ();   
  gtk_widget_show (x);
  gtk_box_pack_start (b, x, TRUE, TRUE, 1);

  if (value)
     gtk_entry_set_text (x, value);

  gtk_entry_set_editable (x, FALSE);

  GtkWidget *bt = gtk_button_new_with_label (_("Choose"));
  gtk_widget_show (bt);
  gtk_box_pack_start (b, bt, FALSE, FALSE, 1);
  g_signal_connect (bt, "clicked", G_CALLBACK (cb_select_font), x);
  
  return x;
}


static void cb_select_color (GtkObject *object, GtkWidget *w) 
{
  GtkEntry *e = w;
  GdkColor color;

  GtkColorSelectionDialog *dlg_color_selector = gtk_color_selection_dialog_new (_("Choose a font"));

  gdk_color_parse (gtk_entry_get_text (e), &color);
  gtk_color_selection_set_current_color (dlg_color_selector->colorsel, &color);    
  
  gchar *t;
  
  if (gtk_dialog_run (dlg_color_selector) == GTK_RESPONSE_OK)
     {
      gtk_color_selection_get_current_color (dlg_color_selector->colorsel, &color);
      t = g_strdup_printf("#%02x%02x%02x", color.red / 256, color.green / 256, color.blue / 256);
      gtk_entry_set_text (e, t);  
      g_free (t);
     }

  gtk_widget_destroy (dlg_color_selector);
}


GtkWidget* tea_color_selector (GtkWidget *container, gchar *caption, gchar *value)
{

  GtkWidget *b = gtk_hbox_new (FALSE, 1);
  gtk_widget_show (b);
  gtk_box_pack_start (container, b, FALSE, TRUE, 1);

  GtkWidget *x = gtk_label_new (caption);  
  gtk_widget_show (x);
  gtk_box_pack_start (b, x, FALSE, FALSE, 1);

  x = gtk_entry_new ();   
  gtk_widget_show (x);
  gtk_box_pack_start (b, x, TRUE, TRUE, 1);

  if (value)
     gtk_entry_set_text (x, value);

  gtk_entry_set_editable (x, FALSE);

  GtkWidget *bt = gtk_button_new_with_label (_("Choose"));
  gtk_widget_show (bt);
  gtk_box_pack_start (b, bt, FALSE, FALSE, 1);

  g_signal_connect (bt, "clicked", G_CALLBACK (cb_select_color), x);
  
  return x;
}


GtkWidget* tea_checkbox (GtkWidget *container, gchar *caption, gboolean *value)
{
  GtkWidget *b = gtk_hbox_new (FALSE, 1);
  gtk_widget_show (b);
  gtk_box_pack_start (container, b, FALSE, TRUE, 1);

  GtkCheckButton *x = gtk_check_button_new_with_label (caption);
  gtk_widget_show (x);
  gtk_box_pack_start (b, x, TRUE, TRUE, 1);
  gtk_toggle_button_set_active (&x->toggle_button, value);
    
  return x;
}


GtkWidget* tea_spinbutton (GtkWidget *container, gchar *caption, gdouble value)
{
  GtkWidget *b = gtk_hbox_new (FALSE, 1);
  gtk_widget_show (b);
  gtk_box_pack_start (container, b, FALSE, TRUE, 1);

  GtkWidget *l = gtk_label_new (caption);  
  gtk_widget_show (l);
  gtk_box_pack_start (b, l, FALSE, FALSE, 1);
 
  GtkCheckButton *x = gtk_spin_button_new  (gtk_adjustment_new  
                                                              (value,
                                                               0.0,
                                                               G_MAXDOUBLE,
                                                               1.0,
                                                               10.0,
                                                               0),

                                            1,
                                            0); 

  gtk_widget_show (x);
  gtk_box_pack_start (b, x, TRUE, TRUE, 1);
    
  return x;
}


void logmemo_set_pos (GtkTextView *v, guint pos)
{
  GtkTextMark  *mark;
  GtkTextIter it;
  GtkTextBuffer *b = gtk_text_view_get_buffer (v);
  gtk_text_buffer_get_iter_at_offset (b, &it, pos);
  gtk_text_buffer_place_cursor (b, &it);
  gtk_text_view_scroll_to_iter (v, &it, 0.0, FALSE, 0.0, 0.0);
}


GList* add_combo_hist_with_def (GtkCombo *c, GList *l, gchar *def, gint max)
{
  if (strlen (gtk_entry_get_text (c->entry)) == 0)
     if (def)
        gtk_entry_set_text (c->entry, def);  

  l = add_to_list_with_limit (l, gtk_entry_get_text (c->entry), max);

  if (l)
    gtk_combo_set_popdown_strings (c, l);

  return l;
}


static void cb_select_dir (GtkObject *object, GtkWidget *w) 
{
  GtkEntry *e = w;

  GtkFileSelection *file_selector = gtk_file_selection_new (_("Choose"));
  gtk_file_selection_set_filename (file_selector, gtk_entry_get_text (e));    
  
  gchar *t;

  if (gtk_dialog_run (file_selector) == GTK_RESPONSE_OK)
     {
      t = g_strconcat (gtk_file_selection_get_filename (file_selector), G_DIR_SEPARATOR_S, NULL);
      gtk_entry_set_text (e, t);  
      g_free (t);
     }

  gtk_widget_destroy (file_selector);
}


GtkWidget* tea_dir_selector (GtkWidget *container, gchar *caption, gchar *value)
{

  GtkWidget *b = gtk_hbox_new (FALSE, 1);
  gtk_widget_show (b);
  gtk_box_pack_start (container, b, FALSE, TRUE, 1);

  GtkWidget *x = gtk_label_new (caption);  
  gtk_widget_show (x);
  gtk_box_pack_start (b, x, FALSE, FALSE, 1);

  x = gtk_entry_new ();   
  gtk_widget_show (x);
  gtk_box_pack_start (b, x, TRUE, TRUE, 1);

  if (value)
     gtk_entry_set_text (x, value);

  gtk_entry_set_editable (x, FALSE);

  GtkWidget *bt = gtk_button_new_with_label (_("Choose"));
  gtk_widget_show (bt);
  gtk_box_pack_start (b, bt, FALSE, FALSE, 1);
  g_signal_connect (bt, "clicked", G_CALLBACK (cb_select_dir), x);
  
  return x;
}
