/***************************************************************************
                          tea_text_document.c  -  description
                             -------------------
    begin                : Mon Dec 1 2003
    copyright            : (C) 2003-2004 by Peter 'Roxton' Semiletov
    email                : tea@list.ru
 ***************************************************************************/

/*
Copyright (C) 1998 Olivier Sessink and Chris Mazuc
Copyright (C) 1999-2003 Olivier Sessink
(c)Oskar Swida <swida@aragorn.pb.bialystok.pl>
Copyright (C) 1999-2001  David A Knight
Copyright (C) 1998, 1999 Alex Roberts, Evan Lawrence
Copyright (C) 2000, 2002 Chema Celorio, Paolo Maggi
Copyright (C)2004 Tarot Osuji <tarot@sdf.lonestar.org>

*/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#include <gdk/gdk.h>
#include <gtk/gtk.h>
#include <glib.h>
#include <stdio.h>
#include <strings.h>
#include <unistd.h>

#include "tea_text_document.h"
#include "tea_defs.h"
#include "callbacks.h"
#include "interface.h"
#include "tea_config.h"
#include "tea_funx.h"
#include "tea_hl.h"



//from Bluefish:: gtk_easy.c, modified by roxton

/**
 * widget_get_string_size:
 * @widget: #GtkWidget* to put the string on
 * @string: #ghcar* with the string
 *
 * This function will calculate the width in pixels from the
 * string passed to it in string, using the font from widget
 *
 * Return value: #gint pixels
 */
gint widget_get_string_size (GtkWidget *w, gchar *s) 
{
  gint r = -1;

  PangoLayout *l = gtk_widget_create_pango_layout (w, s);
  if (l)
     {
      pango_layout_get_pixel_size (l, &r, NULL);
      g_object_unref (G_OBJECT (l));
     }

  return r;
}


//from Bluefish:: document.c
/**
 * This function is taken from gtksourceview
 * Copyright (C) 2001
 * Mikael Hermansson <tyan@linux.se>
 * Chris Phelps <chicane@reninet.com>
 */
static gint textview_calculate_real_tab_width(GtkWidget *textview, gint tab_size) {
	gchar *tab_string;
	gint counter = 0;
	gint tab_width = 0;

	if (tab_size <= 0)
		return 0;

	tab_string = g_malloc (tab_size + 1);
	while (counter < tab_size) {
		tab_string[counter] = ' ';
		counter++;
	}
	tab_string[tab_size] = '\0';
	tab_width =  widget_get_string_size(textview, tab_string);
	g_free(tab_string);
/*	if (tab_width < 0) tab_width = 0;*/
	return tab_width;
}

/**
 * doc_set_tabsize:
 * @doc: a #Tdocument
 * @tabsize: a #gint with the tab size
 *
 * this function will set the textview from doc to use the tabsize
 * described by tabsize
 *
 * Return value: void
 **/
void doc_set_tabsize(t_note_page *doc, gint tabsize) {
	PangoTabArray *tab_array;
	gint pixels = textview_calculate_real_tab_width(GTK_WIDGET(doc->text_view), tabsize);
	//DEBUG_MSG("doc_set_tabsize, tabsize=%d, pixels=%d\n", tabsize, pixels);
	tab_array = pango_tab_array_new (1, TRUE);
	pango_tab_array_set_tab (tab_array, 0, PANGO_TAB_LEFT, pixels);
	gtk_text_view_set_tabs (GTK_TEXT_VIEW (doc->text_view), tab_array);
	pango_tab_array_free(tab_array);
}


 //from Bluefish :: document.c
/* contributed by Oskar Swida <swida@aragorn.pb.bialystok.pl>, with help from the gedit source */
static gboolean doc_textview_expose_event_lcb(GtkWidget * widget, GdkEventExpose * event, gpointer data) {
	GtkTextView *view = (GtkTextView*)widget;
	GdkRectangle rect;
	GdkWindow *win;
	GtkTextIter l_start,l_end,it;
	gint l_top1,l_top2;
	PangoLayout *l;
	gchar *pomstr;
	gint numlines,w,i;

	win = gtk_text_view_get_window(view,GTK_TEXT_WINDOW_LEFT);
	if (win!=event->window) return FALSE;

	gtk_text_view_get_visible_rect(view,&rect);
	gtk_text_view_get_line_at_y(view,&l_start,rect.y,&l_top1);
	gtk_text_view_get_line_at_y(view,&l_end,rect.y+rect.height,&l_top2);
	l = gtk_widget_create_pango_layout(widget,"");

	numlines = gtk_text_buffer_get_line_count(gtk_text_view_get_buffer(view));
	pomstr = g_strdup_printf("%d",MAX(99,numlines));
	pango_layout_set_text(l,pomstr,-1);
	g_free(pomstr);
	pango_layout_get_pixel_size(l,&w,NULL);
	gtk_text_view_set_border_window_size(view,GTK_TEXT_WINDOW_LEFT,w+4);
	it = l_start;
	for(i=gtk_text_iter_get_line(&l_start);i<=gtk_text_iter_get_line(&l_end);i++) {
		gtk_text_iter_set_line(&it,i);
		gtk_text_view_get_line_yrange(view,&it,&w,NULL);
		gtk_text_view_buffer_to_window_coords(view,GTK_TEXT_WINDOW_LEFT,0,w,NULL,&w);

		pomstr = g_strdup_printf("%d",i+1);
		pango_layout_set_text(l,pomstr,-1);

		gtk_paint_layout(widget->style,win,GTK_WIDGET_STATE(widget),FALSE,NULL,widget,NULL,2,w,l);
      g_free(pomstr);
	}
	g_object_unref(G_OBJECT(l));
	return TRUE;
}

/**
 * document_set_line_numbers:
 * @doc: a #Tdocument*
 * @value: a #gboolean
 *
 * Show or hide linenumbers (at the left of the main GtkTextView).
 *
 * Return value: void
 **/
void document_set_line_numbers(t_note_page* doc, gboolean value) {
	if (value) {
		gtk_text_view_set_left_margin(GTK_TEXT_VIEW(doc->text_view),2);
		gtk_text_view_set_border_window_size(GTK_TEXT_VIEW(doc->text_view),GTK_TEXT_WINDOW_LEFT,20);
		g_signal_connect(G_OBJECT(doc->text_view),"expose-event",G_CALLBACK(doc_textview_expose_event_lcb),doc);
	} else {
		gtk_text_view_set_left_margin(GTK_TEXT_VIEW(doc->text_view),0);
		gtk_text_view_set_border_window_size(GTK_TEXT_VIEW(doc->text_view),GTK_TEXT_WINDOW_LEFT,0);
	}
  doc->linenums = value;

}

//from Gedit ::gedit_view.c
// * Copyright (C) 1998, 1999 Alex Roberts, Evan Lawrence
// * Copyright (C) 2000, 2002 Chema Celorio, Paolo Maggi
//mod. by roxton

void text_view_set_colors (t_note_page *doc, GdkColor *background, GdkColor *text,
                           GdkColor *selection, GdkColor *sel_text)
{
  if (background)
     gtk_widget_modify_base (GTK_WIDGET (doc->text_view), GTK_STATE_NORMAL, background);

  if (text)
     gtk_widget_modify_text (GTK_WIDGET (doc->text_view), GTK_STATE_NORMAL, text);

  if (selection)
    {
     gtk_widget_modify_base (GTK_WIDGET (doc->text_view), GTK_STATE_SELECTED, selection);
     gtk_widget_modify_base (GTK_WIDGET (doc->text_view), GTK_STATE_ACTIVE, selection);
    }

  if (sel_text)
    {
     gtk_widget_modify_text (GTK_WIDGET (doc->text_view), GTK_STATE_SELECTED, sel_text);
     gtk_widget_modify_text (GTK_WIDGET (doc->text_view), GTK_STATE_ACTIVE, sel_text);
    }
}


void do_backup (gchar *file_name, gboolean do_check)
{
  if (do_check)
     if (! confile.do_backup)
        return;

  if (! g_file_test (file_name, G_FILE_TEST_EXISTS))
     return;

  gchar *bak = g_strconcat (file_name, ".bak", NULL);
  
  if (copy_file (file_name, bak))
     log_to_memo (_("%s is created"), bak, LM_NORMAL);
  else
      log_to_memo (_("Cannot do backup for %s"), file_name, LM_ERROR);
                       
  g_free (bak);
}


gboolean text_doc_save (t_note_page *doc, char *a_filename)
{
  if (doc->readonly)
     {
      log_to_memo (_("This file is read-only"), NULL, LM_ERROR);
      return FALSE;
     }

  gboolean result;
  gchar *filename = get_l_filename (a_filename);

  if (! filename)
     return FALSE;

  if (is_rtf (filename))
     {
      log_to_memo (_("TEA can't save RTF. Because."), filename, LM_ERROR);
      g_free (filename);
      return FALSE;
     }
 
  if (g_file_test (filename, G_FILE_TEST_EXISTS))
  if (access (filename, W_OK) != 0)
     {
      log_to_memo (_("%s is not writable for you!"), filename, LM_ERROR);
      g_free (filename); 
      return FALSE;
     }

  if (g_file_test (filename, G_FILE_TEST_IS_DIR))
     {
      log_to_memo (_("And how I can save this text file as a DIRECTORY?!"), NULL, LM_ERROR);
      g_free (filename);
      return FALSE;
     }

  if (g_utf8_collate (doc->encoding, "UTF-8") == 0)
      {
       do_backup (filename, TRUE);
       result = doc_save_buffer_to_file (doc->text_buffer, filename);
      }
  else
  if (g_utf8_collate (doc->encoding, CURR_LOCALE) == 0)
     {
      do_backup (filename, TRUE);
      result = doc_save_buffer_to_file_l (doc->text_buffer, filename);
      }
  else
     {
      do_backup (filename, TRUE);
      result = doc_save_buffer_to_file_iconv (doc->text_buffer, filename, doc->encoding);
     }

  if (g_utf8_collate (filename, confile.tea_rc) == 0)
     {
      confile_free ();
      confile_reload();
      doc_update_all();
      update_enc_menu();
      log_to_memo (_("config reloaded"), NULL, LM_NORMAL);
     }

  if (g_utf8_collate (filename, confile.tea_hotkeys) == 0)
     {
      reload_hotkeys ();
      log_to_memo (_("hotkeys reloaded"), NULL, LM_NORMAL);
     }


  if (g_utf8_collate (filename, confile.user_menu_file) == 0)
     {
      reload_usermenu ();
      log_to_memo (_("user menu reloaded"), NULL, LM_NORMAL);
     }

  if (g_utf8_collate (filename, confile.tea_kwas_bookmarks) == 0)
     {
      glist_strings_free (gl_tea_kwas_bookmarks); 
      if (g_file_test (confile.tea_kwas_bookmarks, G_FILE_TEST_EXISTS))
          gl_tea_kwas_bookmarks = load_file_to_glist (confile.tea_kwas_bookmarks);

      log_to_memo (_("kwas bookmarks reloaded"), NULL, LM_NORMAL);
     }


  if (g_utf8_collate (filename, confile.tea_autoreplace) == 0)
      reload_autoreplace ();

  if (g_utf8_collate (filename, confile.iconv_file) == 0)
     {
      get_iconv_sup();
      update_enc_menu();
      log_to_memo (_("%s reloaded"), confile.iconv_file, LM_NORMAL);
     }

  if (g_utf8_collate (filename, confile.bmx_file) == 0)
     {
      bmx_reload ();
      log_to_memo (_("%s reloaded"), confile.bmx_file, LM_NORMAL);
     }

  doc->hl_mode = ch_str (doc->hl_mode, get_hl_name (doc->file_name));

  g_free (filename);

  return result;
}


gchar* doc_get_sel (t_note_page *doc) 
{
  GtkTextIter start;
  GtkTextIter end;
  if (gtk_text_buffer_get_selection_bounds (doc->text_buffer, &start, &end))
     return (gtk_text_buffer_get_text (doc->text_buffer, &start, &end, FALSE));
  else
      return NULL;
}


gboolean doc_is_sel (GtkTextBuffer *text_buffer)
{
  return gtk_text_buffer_get_selection_bounds (text_buffer, NULL, NULL);
}


gchar* doc_get_buf (GtkTextBuffer *text_buffer)
{
  GtkTextIter itstart, itend;

  gtk_text_buffer_get_iter_at_offset (text_buffer, &itstart, 0);
  gtk_text_buffer_get_iter_at_offset(text_buffer, &itend, gtk_text_buffer_get_char_count (text_buffer));

  return gtk_text_buffer_get_text (text_buffer, &itstart, &itend, FALSE);
}


void doc_select_line (t_note_page *doc, gint line)
{
  GtkTextIter itstart, itend;
  gtk_text_buffer_get_iter_at_line (doc->text_buffer, &itstart, line-1);
  itend = itstart;
  if (gtk_text_iter_forward_to_line_end (&itend))
     {
      gtk_text_buffer_move_mark_by_name (doc->text_buffer, "insert", &itstart);
      gtk_text_buffer_move_mark_by_name (doc->text_buffer, "selection_bound", &itend);
      gtk_text_view_scroll_to_iter (GTK_TEXT_VIEW (doc->text_view), &itstart, 0.25, FALSE, 0.5, 0.5);
     }
}


gboolean doc_save_buffer_to_file (GtkTextBuffer *text_buffer, gchar *filename)
{
  gchar *t = doc_get_buf (text_buffer);
  gboolean r = save_string_to_file (filename, t);
  g_free (t);
  return r;
}


gboolean doc_save_buffer_to_file_l (GtkTextBuffer *text_buffer, gchar *filename)
{
  gsize bytes_read;
  gsize bytes_written;
  gchar *buf = doc_get_buf (text_buffer);

  if (! buf)
     return FALSE;

  gchar *lbuf = g_locale_from_utf8 (buf, -1, &bytes_read, &bytes_written, NULL);
  gboolean result = save_string_to_file (filename, lbuf);

  g_free (buf);
  g_free (lbuf);
                        
  return result;
}


gboolean doc_save_buffer_to_file_iconv (GtkTextBuffer *text_buffer, gchar *filename, gchar *enc)
{
  gchar *t;
  gboolean result = FALSE;
  gsize bytes_read;
  gsize bytes_written;

  gchar *buf = doc_get_buf(text_buffer);
  if (! buf)
     return FALSE;

  t  = g_convert (buf, -1, enc, "UTF-8", &bytes_read, &bytes_written, NULL);

  if (! t)
     {
      g_free (buf);
      return FALSE;
     }

  result = save_string_to_file (filename, t);

  g_free (buf);
  g_free (t);

  return result;
}


static const char *xpm_data[] = 
                                {
                                 "6 6 2 1",
                                 "  c None",
                                 ". c #000000000000",
                                 ".    .",
                                 " .  . ",
                                 "  ..  ",
                                 "  ..  ",
                                 " .  . ",
                                 ".    ."
                                };


t_note_page* page_create_new (void)
{
  t_note_page *page = (t_note_page *) g_malloc (sizeof (t_note_page));

  page->position = 1;
  page->readonly = FALSE;
  page->current_path = NULL;
  page->last_searched_text = NULL;
  page->hl_mode = g_strdup (HL_NONE);

  page->scrolledwindow = gtk_scrolled_window_new (NULL, NULL);

  gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (page->scrolledwindow), GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);

  gtk_widget_show (page->scrolledwindow);

  page->text_view = gtk_text_view_new ();

  page->encoding = g_strdup ("UTF-8");
  page->linenums = FALSE;

  gtk_widget_show (page->text_view);
 
  if (confile.word_wrap == 1)
     gtk_text_view_set_wrap_mode (GTK_TEXT_VIEW (page->text_view), GTK_WRAP_WORD);
  else
      gtk_text_view_set_wrap_mode (GTK_TEXT_VIEW (page->text_view), GTK_WRAP_NONE);

  gtk_container_set_border_width (GTK_CONTAINER (page->text_view), 1);

  gtk_container_add (GTK_CONTAINER (page->scrolledwindow), page->text_view);

  page->tab_label = gtk_label_new (NULL);
  GTK_WIDGET_UNSET_FLAGS (page->tab_label, GTK_CAN_FOCUS);

  GtkWidget *hbox;
  hbox = gtk_hbox_new (FALSE, 0);

  page->but = gtk_button_new ();

  GtkWidget *w_pixmap;
  GdkPixmap *pixmap;
  GdkBitmap *mask;
  GtkStyle *style;

  style = gtk_widget_get_style (tea_main_window);
  pixmap = gdk_pixmap_create_from_xpm_d (tea_main_window->window,  &mask,
                                         &style->bg[GTK_STATE_NORMAL],
                                         (gchar **) xpm_data);

  w_pixmap = gtk_image_new_from_pixmap (pixmap, mask);
  gtk_widget_show (w_pixmap);
  gtk_container_add (GTK_CONTAINER (page->but), w_pixmap);

  gtk_box_pack_start (GTK_BOX(hbox), page->tab_label, FALSE, FALSE, 0);
  gtk_box_pack_start (GTK_BOX(hbox), page->but, FALSE, FALSE, 0);
  gtk_widget_show (hbox);
  gtk_widget_show (page->but);
  gtk_widget_show (page->tab_label);
 
  g_signal_connect ((gpointer) page->but, "clicked",
                    G_CALLBACK (on_button_close),
                    page);

  page->b_saved = FALSE;
  page->file_name = g_strdup ("noname");

  gtk_text_view_set_buffer (page->text_view, gtk_text_buffer_new (main_tags_table));

  page->text_buffer = gtk_text_view_get_buffer (page->text_view);

//  g_signal_connect ((gpointer) page->text_view, "destroy-event",
  //                   G_CALLBACK (on_page_kill_page),
    //                 page);

  

  g_signal_connect (page->text_view, "key-press-event",
                     G_CALLBACK (on_editor_keypress), page);
 


  page->um = UNDOMGR (undomgr_new ());
  undomgr_attach (page->um, page->text_view);

  dox = g_list_append (dox, page);
  gtk_notebook_append_page_menu (GTK_NOTEBOOK(notebook1), page->scrolledwindow, hbox, NULL);

  if (confile.show_line_nums == 1)
     document_set_line_numbers (page, TRUE);

  set_dnd_accept (page->text_view);

  return page;
}


gpointer glist_find_by_index (GList *list, int i)
{
  return g_list_nth_data (list, i);
}


t_note_page* get_page (void)
{
  cur_text_doc = NULL;

  gint i = gtk_notebook_get_current_page (notebook1);
  if (i == -1)
    return NULL;
  else
      cur_text_doc = (t_note_page *) g_list_nth_data (dox, i);

  return cur_text_doc;
}


t_note_page* get_page_by_index (int i)
{
  return (t_note_page *) g_list_nth_data (dox, i);
}


void page_del_by_index (int i)
{
  t_note_page *page;

  if (i != -1)
     {
      page = glist_find_by_index (dox, i);
      log_to_memo (_("%s was closed"), page->file_name, LM_NORMAL);
      dox = g_list_remove (dox, page);
      page_free (page);
      gtk_notebook_remove_page (notebook1, i);
      update_recent_list_menu (FALSE);
     }
}


t_note_page* doc_clear_new (void) 
{
  t_note_page *doc = page_create_new ();
  doc->file_name = get_noname_name();
  gtk_label_set_label (doc->tab_label, doc->file_name);
  doc_apply_settings (doc);
  gtk_notebook_set_current_page (notebook1, g_list_length (dox) - 1);
  return doc;
}


t_note_page* doc_open_file (gchar *a_filename)
{
  gsize length;
  GError *error;
  gchar *buf;
  gchar *newlabel;
  gsize bytes_read;
  gsize bytes_written;
  gchar *enc;
  gchar *t;
  gchar *filename;

  t_note_page* page = NULL;
 
  
  if (! g_file_test (a_filename, G_FILE_TEST_EXISTS))
     {
      log_to_memo (_("%s is NOT exists!"), a_filename, LM_ERROR);
      return NULL;       
     }

  
  if (access (a_filename, R_OK) != 0)
     {
      log_to_memo (_("You have NO access to %s"), a_filename, LM_ERROR);
      return NULL;
     }

  filename = g_strdup (a_filename);

  if (g_file_test (a_filename, G_FILE_TEST_IS_SYMLINK))
     {
      g_free (filename);
      filename = (gchar *) g_malloc (2048);
      readlink (a_filename, filename, 2048);
     
      if (access (filename, R_OK) != 0)
        {
         log_to_memo (_("You have NOT access to %s"), filename, LM_ERROR);
         g_free (filename);
         return NULL;
        }
     }

   if (is_rtf (filename))
      {
       buf = text_load_rtf (filename);
       page = doc_ins_to_new (buf); 
       g_free (buf);
       g_free (page->file_name);

       page->file_name = get_8_filename (filename);

       page->b_saved = TRUE;
 
       newlabel = g_path_get_basename (page->file_name);
       gtk_label_set_label (page->tab_label, newlabel);
                
       doc_apply_settings (page);
       editor_set_pos (page, 0);

       log_to_memo (_("%s is opened"), page->file_name, LM_NORMAL);
       g_free (filename);
       g_free (newlabel);
       tabs_reload ();
     
       return page;
      }   

  if (! g_file_get_contents (filename, &buf, &length, &error))
      {
       g_free (filename);
       return NULL;  
      }

  page = doc_clear_new ();
   
  if (page->encoding)
     g_free (page->encoding);

  if (cur_settings.selected_enc)
      page->encoding = g_strdup (cur_settings.selected_enc);
  else
      page->encoding = g_strdup ("UTF-8");

  if (confile.do_enc_autodetect)
  if (g_strncasecmp (page->encoding, "autodetect", 33) == 0)
     {
      enc = enc_guess (buf);
      if (enc)
         {
          g_free (page->encoding);
          page->encoding = g_strdup (enc);
         }
     }
    
  if (g_strncasecmp (page->encoding, "UTF-8", 33) == 0)
     {
                
      if (! g_utf8_validate (buf, -1, NULL))
          {
           g_free (buf);
           page_del_by_index (gtk_notebook_get_current_page ((GtkNotebook *) notebook1));
           log_to_memo (_("Wrong encoding selected, try again with another one."), NULL, LM_ERROR);
           page = query_enc (filename);
           g_free (filename);
           tabs_reload ();  
           return page;
          }

       gtk_text_buffer_set_text (page->text_buffer, buf, length);
       
      }
  else
  if (g_strncasecmp (page->encoding, CURR_LOCALE, 33) == 0)
     {
      t = g_locale_to_utf8 (buf, length, &bytes_read, &bytes_written, &error);

      if (! t)
         {
          g_free (buf);
          page_del_by_index (gtk_notebook_get_current_page ((GtkNotebook *) notebook1));
          log_to_memo (_("Wrong encoding selected, try again with another one."), NULL, LM_ERROR);
          page = query_enc (filename);
          g_free (filename);
          tabs_reload ();
          return page;
         }

      gtk_text_buffer_set_text (page->text_buffer, t, bytes_written);
 
      g_free (t);
     }
  else
      { 
       t = g_convert (buf, length, "UTF-8", page->encoding, &bytes_read, &bytes_written, &error);
       if (! t)
          {
           page_del_by_index (gtk_notebook_get_current_page ((GtkNotebook *) notebook1));
           log_to_memo (_("Wrong encoding selected, try again with another one."), NULL, LM_ERROR);
           page = query_enc (filename);
           g_free (filename);
           g_free (buf);
           tabs_reload ();
           return page;
          }

       gtk_text_buffer_set_text (page->text_buffer, t, bytes_written);
       g_free (t);
      }

  g_free (page->file_name);
  page->file_name = get_8_filename (filename);

  page->b_saved = TRUE;
 
  newlabel = g_path_get_basename (page->file_name);
  gtk_label_set_label (page->tab_label, newlabel);
                
  doc_apply_settings (page);
  editor_set_pos (page, 0);

  page->hl_mode = ch_str (page->hl_mode, get_hl_name (page->file_name));
  
  log_to_memo (_("%s is opened"), page->file_name, LM_NORMAL);

  if (confile.scan_for_links_on_doc_open == 1)
      scan_links();

  if (confile.do_hl_on_fileopen)
     apply_hl (page);

  g_free (filename);
  g_free (newlabel);
  g_free (buf);

  tabs_reload ();

  return page;
}


void doc_apply_settings (t_note_page *doc)
{
  if (! doc) return;
  
  PangoFontDescription *font_desc = pango_font_description_from_string (confile.editor_font);
  gtk_widget_modify_font (doc->text_view, font_desc);
  pango_font_description_free (font_desc);

  g_object_set (G_OBJECT (tag_comment), "foreground", confile.tag_comment, "font", confile.tag_comment_font, NULL);
  g_object_set (G_OBJECT (tag_identifier), "foreground", confile.tag_identifier, "font", confile.tag_identifier_font, NULL);
  g_object_set (G_OBJECT (tag_digit), "foreground", confile.tag_digit, "font", confile.tag_digit_font, NULL);
  g_object_set (G_OBJECT (tag_string), "foreground", confile.tag_string, "font", confile.tag_string_font, NULL);
  g_object_set (G_OBJECT (tag_html_tag), "foreground", confile.tag_html_tag, "font", confile.tag_html_tag_font, NULL);
  g_object_set (G_OBJECT (tag_preprocessor), "foreground", confile.tag_preprocessor, "font", confile.tag_preprocessor_font, NULL);
  g_object_set (G_OBJECT (tag_type), "foreground", confile.tag_type, "font", confile.tag_type_font, NULL);

  GdkColor background;
  GdkColor text;
  GdkColor selection;
  GdkColor sel_text;

  gdk_color_parse (confile.text_foreground, &text);
  gdk_color_parse (confile.text_background, &background);
  gdk_color_parse (confile.text_sel_background, &selection);
  gdk_color_parse (confile.text_sel_foreground, &sel_text);
  
  text_view_set_colors (doc, &background, &text, &selection, &sel_text);  
  doc_set_tabsize (doc, confile.tab_size);
}

/*
static gboolean on_page_kill_page (GtkWidget *widget,
                            GdkEvent *event,
                            gpointer user_data)
{
  //t_note_page *page = user_data;
  dbm ("on_page_kill_page");
//  undomgr_detach (UNDOMGR (page->um), GTK_TEXT_VIEW (widget));
 
  return TRUE;

}
*/

           
void page_free (t_note_page *page)
{
  if (page->text_view)
     undomgr_reset (page->um);

  gchar *f = get_l_filename (page->file_name);

  if (strcmp (f, confile.crapbook_file) == 0)
      doc_save_buffer_to_file (page->text_buffer, confile.crapbook_file); 
  else 
      if (g_file_test (f, G_FILE_TEST_EXISTS))
          add_recent_internal (page);

  g_free (f); 

  g_free (page->last_searched_text);
  g_free (page->encoding);
  g_free (page->hl_mode);
  g_free (page->file_name);
  g_free (page->current_path);
  g_free (page);

  tabs_reload ();
}


void doc_insert_at_cursor (t_note_page* doc, gchar *text)
{
  if (! text)
     return;
  gtk_text_buffer_begin_user_action (doc->text_buffer);
  gtk_text_buffer_insert_at_cursor (doc->text_buffer, text, -1);
  gtk_text_buffer_end_user_action (doc->text_buffer);
}

      
void doc_update_cb (gpointer data, gpointer user_data)
{
  if (data)
     doc_apply_settings ((t_note_page*) data);
}


void doc_update_all (void)
{
  g_list_foreach (dox, doc_update_cb, NULL);
}


gint editor_get_pos (t_note_page* doc)
{
  if (! doc->text_buffer && ! doc->text_view)
     return 0;

  GtkTextMark *mark;
  GtkTextIter it;
  mark = gtk_text_buffer_get_mark (doc->text_buffer, "insert");
  gtk_text_buffer_get_iter_at_mark (doc->text_buffer, &it, mark);
  return gtk_text_iter_get_offset (&it);
}


void tv_logmemo_set_pos (guint pos )
{
  GtkTextIter it;
  gtk_text_buffer_get_iter_at_offset (log_memo_textbuffer, &it, pos);
  gtk_text_buffer_place_cursor (log_memo_textbuffer, &it);
  gtk_text_view_scroll_to_iter (tv_logmemo, &it, 0.0, FALSE, 0.0, 0.0);
}


gboolean doc_reload_text (t_note_page *doc, gchar *filename, gchar *enc)
{
  gsize *length;
  GError *error;
  gchar *buf;
  gchar *t;
  gsize bytes_read;
  gsize bytes_written;

  if (! g_file_get_contents (filename, &buf, &length, &error))
     return FALSE;
  else
  if (g_strncasecmp (enc, CURR_LOCALE, 33) == 0)
     {
      gchar *t = g_locale_to_utf8 (buf, length, &bytes_read, &bytes_written, NULL);

      if (! t)
         {
          g_free (buf);
          return FALSE;
         }

      gtk_text_buffer_set_text (doc->text_buffer, t, bytes_written);

      g_free (buf);
      g_free (t);

      return TRUE;
     }
  
  else
      if ((g_ascii_strncasecmp (enc, "UTF-8", 33) == 0))
        {
         if (! g_utf8_validate (buf, -1, NULL))
            {
             g_free (buf);
             return FALSE;
            }

         gtk_text_buffer_set_text (doc->text_buffer, buf, length);

         g_free (buf);
         return TRUE;
        }
       else
           {  //not UTF-8
            t = g_convert (buf, length, "UTF-8", enc, &bytes_read, &bytes_written, &error);
            if (! t)
               {
                g_free (buf);
                return FALSE;
               }

            gtk_text_buffer_set_text (doc->text_buffer, t, bytes_written);

            g_free (buf);
            g_free (t);
            
            return TRUE;
      }
}


gint get_n_page_by_filename (gchar* filename)
{
  gint i = 0;
  t_note_page *t;

  GList *tl = g_list_first (dox);

  while (tl)
        {
         t = (t_note_page*) tl->data;
         if (strcmp (filename, t->file_name) == 0)
            return i;
         i++;
       
         tl = g_list_next (tl);
        }

  return -1;
}


static gint log_to_memo_counter = 0;

void log_to_memo (gchar *m1, gchar* m2, gint mode)
{
  if (! log_memo_textbuffer)
     return;

  if ((++log_to_memo_counter) == confile.logmemo_lines_max)
     {
      log_to_memo_counter = 0;
      GtkTextIter itstart, itend;
      gtk_text_buffer_get_bounds (log_memo_textbuffer, &itstart, &itend);   
      gtk_text_buffer_delete (log_memo_textbuffer, &itstart, &itend);
     }

  if (confile.msg_counter == G_MAXINT)
     confile.msg_counter = 0;

  gchar *prefix = g_strdup_printf  ("(%d) ", ++confile.msg_counter);
  gchar *st;

  if (! m2)
     {
      st = g_strconcat (prefix, m1, "\n", NULL);

      GtkTextIter it;
      gtk_text_buffer_get_iter_at_offset (log_memo_textbuffer, &it, 0);
      gtk_text_buffer_place_cursor (log_memo_textbuffer, &it);

      if (mode == LM_NORMAL)
          gtk_text_buffer_insert_with_tags_by_name (log_memo_textbuffer, &it, st, -1, "lm_normal", NULL);
      else
          if (mode == LM_ERROR)
             gtk_text_buffer_insert_with_tags_by_name (log_memo_textbuffer, &it, st, -1, "lm_error", NULL);
      else
          if (mode == LM_ADVICE)
             gtk_text_buffer_insert_with_tags_by_name (log_memo_textbuffer, &it, st, -1, "lm_advice", NULL);
      else
          if (mode == LM_GREET)
             gtk_text_buffer_insert_with_tags_by_name (log_memo_textbuffer, &it, st, -1, "lm_greet", NULL);

      g_free (st);
      g_free (prefix);
      tv_logmemo_set_pos (0);
      return;
     }

 
  gulong len = strlen (m1) + strlen (m2) + 2024;
  gchar *message = g_malloc (len);

  g_snprintf (message, len, m1, m2);
  gchar *s = g_strconcat (prefix, message, "\n", NULL);
  
  gtk_text_buffer_insert_at_cursor (log_memo_textbuffer, s, -1);

  g_free (message);
  g_free (s);
  g_free (prefix);
  tv_logmemo_set_pos (0);
  
  return;
}


gboolean find_space (gunichar ch, gpointer user_data)
{
  if (g_unichar_isspace (ch) || (ch == '\0')) 
     return TRUE;
  else
      return FALSE; 
}


gboolean find_quote (gunichar ch, gpointer user_data)
{
  if ((ch == '"') || (ch == '\'')) 
     return TRUE;
  else
      return FALSE; 
}


//n.p. Nirvana - I Hate Myself And I Want To Die

gchar* doc_get_current_word (t_note_page *doc, GtkTextIter *itstart, GtkTextIter *itend) 
{
  GtkTextIter ittemp;
  GtkTextMark *mark = gtk_text_buffer_get_insert (doc->text_buffer);
  gtk_text_buffer_get_iter_at_mark (doc->text_buffer, &ittemp, mark);

  gunichar c;
  
  *itstart = ittemp;
  *itend = ittemp;

  if (! gtk_text_iter_forward_find_char (itend, find_space, NULL, NULL))
      gtk_text_buffer_get_end_iter (doc->text_buffer, itend);

  if (! gtk_text_iter_backward_find_char (itstart, find_space, NULL, NULL))
     gtk_text_buffer_get_start_iter (doc->text_buffer, itstart); 
    else     
       gtk_text_iter_forward_char (itstart);  
         
  return (gtk_text_buffer_get_text (doc->text_buffer, itstart, itend, FALSE));
}


void doc_set_new_text (t_note_page *doc, GtkTextIter *itstart, GtkTextIter *itend, gchar *text) 
{
  gtk_text_buffer_begin_user_action(doc->text_buffer);

  gtk_text_buffer_delete (doc->text_buffer, itstart, itend);   
  gtk_text_buffer_insert (doc->text_buffer, itstart, text, -1);

  gtk_text_buffer_end_user_action(doc->text_buffer);
}


void doc_rep_sel (t_note_page *page, const gchar *newstring)
{
  if (! newstring)
     return; 

  GtkTextIter start;
  GtkTextIter end;
  if (gtk_text_buffer_get_selection_bounds (page->text_buffer, &start, &end))
     doc_set_new_text (page, &start, &end, newstring);  
}


gchar* get_c_url (t_note_page *doc) 
{
  GtkTextIter ittemp;
  GtkTextIter itstart;
  GtkTextIter itend;
  gchar *filename;

  GtkTextMark *mark = gtk_text_buffer_get_insert (doc->text_buffer);
  gtk_text_buffer_get_iter_at_mark (doc->text_buffer, &ittemp, mark);

  itstart = ittemp;
  itend = ittemp;

  if (! gtk_text_iter_forward_find_char (&itend, find_quote, NULL, NULL))
      gtk_text_buffer_get_end_iter (doc->text_buffer, &itend);

  if (! gtk_text_iter_backward_find_char (&itstart, find_quote, NULL, NULL))
     gtk_text_buffer_get_start_iter (doc->text_buffer, &itstart); 
    else     
       gtk_text_iter_forward_char (&itstart);  

  gchar *result = gtk_text_buffer_get_text (doc->text_buffer, &itstart, &itend, FALSE);

  if (g_utf8_get_char (result) == '#')
     {
      goto_local_label (result);
      g_free (result);
      return NULL;
     }

  gchar *dir = g_path_get_dirname (doc->file_name);

  if (result [0] == G_DIR_SEPARATOR)
      filename = create_full_path ((result+1), dir);
  else
      filename = create_full_path (result, dir);

  g_free (result);
  gchar *result2 = strdup (filename);
  g_free (filename);

  return result2;
}


gboolean doc_search_f (t_note_page *doc, const gchar *text) 
{
  if (! text)
      return FALSE; 

  GtkTextIter ittemp;
  GtkTextIter end;
  GtkTextIter match_start;
  GtkTextIter match_end;
  gboolean result = FALSE;


  gtk_text_buffer_get_bounds (doc->text_buffer, &ittemp, &end);

  if (gtk_text_iter_forward_search (&ittemp, text, GTK_TEXT_SEARCH_TEXT_ONLY, &match_start, &match_end, &end))
     {
      gtk_text_buffer_move_mark_by_name (doc->text_buffer, "insert", &match_end);
      gtk_text_buffer_move_mark_by_name (doc->text_buffer, "selection_bound", &match_start);
      gtk_text_view_scroll_to_iter (doc->text_view, &match_start, 0.0, TRUE, 0.0, 0.0 );
      g_free (doc->last_searched_text);
      doc->last_searched_text = g_strdup (text); 
      result = TRUE;
     }

   return result;
}


gboolean doc_search_f_next (t_note_page *doc) 
{
  gboolean result = FALSE;
  if (! doc->last_searched_text)
     return FALSE;

  GtkTextIter ittemp;
  GtkTextIter end;
  GtkTextIter match_start;
  GtkTextIter match_end;
  GtkTextMark *mark = gtk_text_buffer_get_insert (doc->text_buffer);
  gtk_text_buffer_get_iter_at_mark (doc->text_buffer, &ittemp, mark);

  gtk_text_buffer_get_end_iter (doc->text_buffer, &end); 

  if (gtk_text_iter_forward_search (&ittemp, doc->last_searched_text, GTK_TEXT_SEARCH_TEXT_ONLY, &match_start, &match_end, &end))
     {
      gtk_text_buffer_move_mark_by_name (doc->text_buffer, "insert", &match_end);
      gtk_text_buffer_move_mark_by_name (doc->text_buffer, "selection_bound", &match_start);
      gtk_text_view_scroll_to_iter (doc->text_view, &match_start, 0.0, TRUE, 0.0, 0.0 );
      result = TRUE;
     }

  return result;
}


void doc_replace_all (t_note_page *doc, const gchar *text, const gchar *new_text) 
{
  if (! text)
      return;
 
  if (! new_text)
      return;

  if (doc_search_f (doc, text))
     {
      doc_rep_sel (doc, new_text);
      while (doc_search_f_next (doc))
             doc_rep_sel (doc, new_text);
     }  
}


gint find_index_by_page (t_note_page *page)
{
 gint i = -1;

 GList *p = g_list_first (dox);

 while (p)
       {
        i++;  
        if (page == p->data)
           return i;

        p = g_list_next (p);  
       }

 return i;
}


gchar* get_c_url_pure (t_note_page *doc) 
{
  GtkTextIter ittemp;
  GtkTextIter itstart;
  GtkTextIter itend;

  GtkTextMark *mark = gtk_text_buffer_get_insert (doc->text_buffer);
  gtk_text_buffer_get_iter_at_mark (doc->text_buffer, &ittemp, mark);

  itstart = ittemp;
  itend = ittemp;

  if (! gtk_text_iter_forward_find_char (&itend, find_quote, NULL, NULL))
      gtk_text_buffer_get_end_iter (doc->text_buffer, &itend);

  if (! gtk_text_iter_backward_find_char (&itstart, find_quote, NULL, NULL))
     gtk_text_buffer_get_start_iter (doc->text_buffer, &itstart); 
    else     
       gtk_text_iter_forward_char (&itstart);  
  
  gchar *result = gtk_text_buffer_get_text (doc->text_buffer, &itstart, &itend, FALSE);

  return result;
}


t_note_page* doc_ins_to_new (const gchar *newstring)
{
  t_note_page *page = doc_clear_new ();
  gtk_window_set_title (GTK_WINDOW (tea_main_window), page->file_name);
  gtk_widget_grab_focus (page->text_view);
  if (newstring)
     doc_insert_at_cursor (page, newstring);
  tabs_reload ();
  return page;
}


t_note_page* get_page_text (void)
{
  cur_text_doc = NULL;

  gint i = gtk_notebook_get_current_page (notebook1);
  if (i == -1)
    return NULL;
  else
      cur_text_doc = (t_note_page *) g_list_nth_data (dox, i);


  return cur_text_doc;
}


void set_title (t_note_page *t)
{
  if (! t)
     return;

  if (! t->file_name)
     return;      

  if (! g_utf8_validate (t->file_name, strlen (t->file_name), NULL))
      return;

  gchar *nc = NULL;

  if (confile.main_wnd_show_full_path)
      nc = g_strdup (t->file_name);
  else 
      nc = g_path_get_basename (t->file_name);

  gtk_window_set_title (GTK_WINDOW (tea_main_window), nc);
  statusbar_msg (t->encoding);

  g_free (nc);
}
  

t_note_page* open_file_std (gchar *f)
{
  cur_text_doc = doc_open_file (f);
  tabs_reload ();
  set_title (cur_text_doc);
  return cur_text_doc;
}


void doc_move_to_pos_bw (t_note_page* doc, gint pos)
{
  if (! doc->text_buffer && ! doc->text_view)
     return 0;

  GtkTextMark *mark;
  GtkTextIter it;
  mark = gtk_text_buffer_get_mark (doc->text_buffer, "insert");
  gtk_text_buffer_get_iter_at_mark (doc->text_buffer, &it, mark);
  
  if (gtk_text_iter_backward_chars (&it, pos))
      gtk_text_buffer_place_cursor (doc->text_buffer, &it);
  }


//n.p. Albinoni - Adagio G-moll
void doc_move_to_pos_bw_quote (t_note_page* doc)
{
  if (! doc->text_buffer && ! doc->text_view)
     return 0;

  GtkTextMark *mark;
  GtkTextIter it;
  mark = gtk_text_buffer_get_mark (doc->text_buffer, "insert");
  gtk_text_buffer_get_iter_at_mark (doc->text_buffer, &it, mark);
  
  if (gtk_text_iter_backward_find_char (&it, find_quote, NULL, NULL))
      gtk_text_buffer_place_cursor (doc->text_buffer, &it);
     
}


void editor_set_pos (t_note_page *doc, guint pos)
{
  GtkTextIter it;
  GtkTextMark* m; 

  gtk_text_buffer_get_iter_at_offset (doc->text_buffer, &it, pos);
  gtk_text_buffer_place_cursor (doc->text_buffer, &it);
  m = gtk_text_buffer_get_mark (doc->text_buffer, "insert");
  gtk_text_view_scroll_to_mark (doc->text_view, m, 0.0, TRUE, 0.0, 0.0);
}
 

void editor_goto_selection (t_note_page *doc)
{
  GtkTextIter start;
  GtkTextIter end;

  if (gtk_text_buffer_get_selection_bounds (doc->text_buffer, &start, &end))
      gtk_text_view_scroll_to_iter (doc->text_view, &start, 0.0, TRUE, 0.0, 0.0); 

}


gboolean doc_search_f_ncase (t_note_page *doc, const gchar *text) 
{
  if (! text)
     return FALSE;

  gboolean result = FALSE;

  gchar *f = g_utf8_strdown (text, -1);
 
  GtkTextIter start;
  GtkTextIter end;
  GtkTextIter ittemp;
  GtkTextIter b_end;

  GtkTextMark *mark = gtk_text_buffer_get_insert (doc->text_buffer);
  gtk_text_buffer_get_iter_at_mark (doc->text_buffer, &ittemp, mark);

  gtk_text_iter_forward_chars (&ittemp, g_utf8_strlen (f, -1));
  
  gint int_char_offset = gtk_text_iter_get_offset (&ittemp);

  gtk_text_buffer_get_end_iter (doc->text_buffer, &b_end);


  gchar *t = gtk_text_buffer_get_slice (doc->text_buffer,
                                        &ittemp,
                                        &b_end,
                                        FALSE);

    gchar *x = g_utf8_strdown (t, -1);


  gchar *found = strstr (x, f);
 
  if (found)
     {
      glong i = g_utf8_pointer_to_offset (x, found) + int_char_offset;
  
      gtk_text_buffer_get_iter_at_offset (doc->text_buffer, &start, i);

      gtk_text_buffer_get_iter_at_offset (doc->text_buffer, &end,
                                          i+ g_utf8_strlen (f, -1));


      gtk_text_buffer_move_mark_by_name (doc->text_buffer, "insert", &start);
      gtk_text_buffer_move_mark_by_name (doc->text_buffer, "selection_bound", &end);
      g_free (doc->last_searched_text);
      doc->last_searched_text = g_strdup (text); 
      result = TRUE;
     }     

  g_free (t);
  g_free (x);
  g_free (f);

  return result;
}


gboolean doc_search_f_next_ncase (t_note_page *doc) 
{
  if (! doc->last_searched_text)
      return FALSE;

  GtkTextIter start;
  GtkTextIter end;

  gchar *f = g_utf8_strdown (doc->last_searched_text, -1);

  gboolean result = FALSE;
  GtkTextIter ittemp;
  GtkTextIter b_start;
  GtkTextIter b_end;
  GtkTextMark *mark = gtk_text_buffer_get_insert (doc->text_buffer);
  gtk_text_buffer_get_iter_at_mark (doc->text_buffer, &ittemp, mark);

  gtk_text_iter_forward_chars (&ittemp, g_utf8_strlen (f, -1));
  
  gint int_char_offset = gtk_text_iter_get_offset (&ittemp);

  gtk_text_buffer_get_end_iter (doc->text_buffer, &b_end);


  gchar *t = gtk_text_buffer_get_slice (doc->text_buffer,
                                        &ittemp,
                                        &b_end,
                                        FALSE);

  gchar *x = g_utf8_strdown (t, -1);
  gchar *found = strstr (x, f);

  if (found)
     {
      glong i = g_utf8_pointer_to_offset (x, found) + int_char_offset;
  
      gtk_text_buffer_get_iter_at_offset (doc->text_buffer, &start, i);
      gtk_text_buffer_get_iter_at_offset (doc->text_buffer, &end, i + g_utf8_strlen (f, -1));
      gtk_text_buffer_move_mark_by_name (doc->text_buffer, "insert", &start);
      gtk_text_buffer_move_mark_by_name (doc->text_buffer, "selection_bound", &end);

      result = TRUE;
     }     

  g_free (t);
  g_free (x);
  g_free (f);
  return result;
}


gchar* upcase_each_first_letter_ofw (t_note_page *doc)
{
  gunichar u;
  
  GtkTextIter start;
  GtkTextIter end;

  if (! gtk_text_buffer_get_selection_bounds (doc->text_buffer, &start, &end))
      return NULL; 
 
  GString* gs = g_string_sized_new (1024);

  do
    {
     if (gtk_text_iter_equal (&start, &end))
        break;

     u = gtk_text_iter_get_char (&start);

     if ( gtk_text_iter_starts_word (&start))        
        u = g_unichar_toupper (u); 
     
     gs = g_string_append_unichar (gs, u);        

    }  
   while ( gtk_text_iter_forward_char (&start) );

  gchar *s = gs->str;
  g_string_free (gs, FALSE);

  return s;
}


gchar* doc_get_word_at_left (t_note_page *doc, GtkTextIter *itstart, GtkTextIter *itend) 
{
  GtkTextIter ittemp;
  GtkTextMark *mark = gtk_text_buffer_get_insert (doc->text_buffer);
  gtk_text_buffer_get_iter_at_mark (doc->text_buffer, &ittemp, mark);
  
  *itstart = ittemp;
  *itend = ittemp;
  
  if (! gtk_text_iter_backward_find_char (itstart, find_space, NULL, NULL))
     gtk_text_buffer_get_start_iter (doc->text_buffer, itstart); 
  else     
      gtk_text_iter_forward_char (itstart);  
         
  return (gtk_text_buffer_get_text (doc->text_buffer, itstart, itend, FALSE));
}


////////////////////
/*
from Leafpad::indent.c
(C)2004 Tarot Osuji <tarot@sdf.lonestar.org>

*/

static gchar *compute_indentation(GtkTextBuffer *buffer, gint line) // from gedit
{
	GtkTextIter start_iter, end_iter;
	gunichar ch;
	
	gtk_text_buffer_get_iter_at_line(buffer, &start_iter, line);
	end_iter = start_iter;
	ch = gtk_text_iter_get_char(&end_iter);
	while (g_unichar_isspace(ch) && ch != '\n') {
		if (!gtk_text_iter_forward_char(&end_iter))
			break;
		ch = gtk_text_iter_get_char(&end_iter);
	}
	if (gtk_text_iter_equal(&start_iter, &end_iter))
		return NULL;
	
	return gtk_text_iter_get_text(&start_iter, &end_iter);
}

void indent_real(GtkWidget *text_view)
{
	GtkTextIter iter;
	gchar *ind, *str;
	
	GtkTextBuffer *buffer = gtk_text_view_get_buffer(GTK_TEXT_VIEW(text_view));
	
	gtk_text_buffer_get_iter_at_mark(buffer, &iter, gtk_text_buffer_get_insert(buffer));
	ind = compute_indentation(buffer, gtk_text_iter_get_line(&iter));
	str = g_strconcat("\n", ind, NULL);
        gtk_text_buffer_insert(buffer, &iter, str, -1);
	g_free(str);
	g_free(ind);
	
	gtk_text_view_scroll_mark_onscreen(
		GTK_TEXT_VIEW(text_view),
		gtk_text_buffer_get_insert(buffer));
}
///////////////////////////////





//from Bluefish::document.c
/**
 * doc_toggle_highlighting_cb:
 * @doc: a #Tdocument*
 * @unindent: #gboolean
 *
 * Indent the selected block in current document.
 * Set unindent to TRUE to unindent.
 *
 * Return value: void
 **/
void doc_indent_selection (t_note_page *doc, gboolean unindent)
 {
	GtkTextIter itstart,itend;
	if (gtk_text_buffer_get_selection_bounds(doc->text_buffer,&itstart,&itend)) {
		GtkTextMark *end;
/*		gboolean firstrun=TRUE;*/

		//doc_unbind_signals(doc);
		//doc_unre_new_group(doc);
		/* we have a selection, now we loop trough the characters, and for every newline
		we add or remove a tab, we set the end with a mark */
		end = gtk_text_buffer_create_mark(doc->text_buffer,NULL,&itend,TRUE);
		if (gtk_text_iter_get_line_offset(&itstart)>0) {
			gtk_text_iter_set_line_index(&itstart,0);
		}	
		while(gtk_text_iter_compare(&itstart,&itend) < 0) {
			GtkTextMark *cur;
/*			if (firstrun && !gtk_text_iter_starts_line(&itstart)) {
				gtk_text_iter_forward_line(&itstart);
			}
			firstrun = FALSE;*/
			cur = gtk_text_buffer_create_mark(doc->text_buffer,NULL,&itstart,TRUE);
			if (unindent) {
				/* when unindenting we try to set itend to the end of the indenting step
				which might be a tab or 'tabsize' spaces, then we delete that part */
				gboolean cont=TRUE;
				gchar *buf = NULL;
				gunichar cchar = gtk_text_iter_get_char(&itstart);
				if (cchar == 9) { /* 9 is ascii for tab */
					itend = itstart;
					cont = gtk_text_iter_forward_char(&itend);
					buf = g_strdup("\t");
				} else if (cchar == 32) { /* 32 is ascii for space */
					gchar *tmpstr;
					gint i=0;
					itend = itstart;
					gtk_text_iter_forward_chars(&itend,confile.tab_size);
					tmpstr = gtk_text_buffer_get_text(doc->text_buffer,&itstart,&itend,FALSE);
					//DEBUG_MSG("tab_width=%d, strlen(tmpstr)=%d, tmpstr='%s'\n",main_v->props.editor_tab_width,strlen(tmpstr),tmpstr);
					while (cont && tmpstr[i] != '\0') {
						cont = (tmpstr[i] == ' ');
						//DEBUG_MSG("doc_indent_selection, tmpstr[%d]='%c'\n",i,tmpstr[i]);
						i++;
					}
					if (cont) {
						buf = tmpstr;
					} else {
						g_free(tmpstr);
					}
				} else {
					cont = FALSE;
				}
				if (cont) {
					gint offsetstart, offsetend;				
					offsetstart = gtk_text_iter_get_offset(&itstart);
					offsetend = gtk_text_iter_get_offset(&itend);
					gtk_text_buffer_delete(doc->text_buffer,&itstart,&itend);
					//doc_unre_add(doc, buf, offsetstart, offsetend, UndoDelete);
				}
#ifdef DEBUG
				else {
					//DEBUG_MSG("doc_indent_selection, NOT continue!!\n");
				}
#endif
			} else { /* indent */
				gint offsetstart = gtk_text_iter_get_offset(&itstart);
				gchar *indentstring;
				gint indentlen;
				if (confile.ins_spaces_on_tab_press) {
                                          indentstring = g_strnfill (confile.tab_size, ' '); 
                  
					//indentstring = bf_str_repeat(" ", confile.tab_size);
					indentlen = confile.tab_size;
				} else {
					indentstring = g_strdup("\t");
					indentlen=1;
				}
				gtk_text_buffer_insert(doc->text_buffer,&itstart,indentstring,indentlen);
				//doc_unre_add(doc, indentstring, offsetstart, offsetstart+indentlen, UndoInsert);
				g_free(indentstring);
			}
			gtk_text_buffer_get_iter_at_mark(doc->text_buffer,&itstart,cur);
			gtk_text_buffer_get_iter_at_mark(doc->text_buffer,&itend,end);
			gtk_text_buffer_delete_mark(doc->text_buffer,cur);
			gtk_text_iter_forward_line(&itstart);
			//DEBUG_MSG("doc_indent_selection, itstart at %d, itend at %d\n",gtk_text_iter_get_offset(&itstart),gtk_text_iter_get_offset(&itend));
		}
		gtk_text_buffer_delete_mark(doc->text_buffer,end);
		//doc_bind_signals(doc);
		//doc_set_modified(doc, 1);
	} else {
		/* there is no selection, work on the current line */
		GtkTextIter iter;
		gtk_text_buffer_get_iter_at_mark(doc->text_buffer,&iter,gtk_text_buffer_get_insert(doc->text_buffer));
		gtk_text_iter_set_line_offset(&iter,0);
		if (unindent) {
			gint deletelen = 0;
			gchar *tmpstr, *tmp2str;
			GtkTextIter itend = iter;
			gtk_text_iter_forward_chars(&itend,confile.tab_size);
			tmpstr = gtk_text_buffer_get_text(doc->text_buffer,&iter,&itend,FALSE);
			//tmp2str = bf_str_repeat(" ", main_v->props.editor_tab_width);
                        tmp2str = g_strnfill  (confile.tab_size, ' '); 
			if (tmpstr[0] == '\t') {
				deletelen = 1;
			} else if (tmpstr && strncmp(tmpstr,tmp2str,confile.tab_size)==0) {
				deletelen = confile.tab_size;
			}
			g_free(tmpstr);
			g_free(tmp2str);
			if (deletelen) {
				itend = iter;
				gtk_text_iter_forward_chars(&itend,deletelen);
				gtk_text_buffer_delete(doc->text_buffer,&iter,&itend);
			}
		} else { /* indent */
			gchar *indentstring;
			gint indentlen;
			if (confile.ins_spaces_on_tab_press) {
                                indentstring = g_strnfill  (confile.tab_size, ' '); 
				//indentstring = bf_str_repeat(" ", main_v->props.editor_tab_width);
				indentlen = confile.tab_size;
			} else {
				indentstring = g_strdup("\t");
				indentlen=1;
			}
			gtk_text_buffer_insert(doc->text_buffer,&iter,indentstring,indentlen);
			g_free(indentstring);
		}
	}
}

